#pragma once

#include <QObject>
#include <QString>
#include <QJsonDocument>
#include <QNetworkAccessManager>

#ifdef Q_OS_WIN
#undef DELETE
#endif

static const uint32_t CURLE_COULDNT_CONNECT = 0;

class RestClient : public QObject {
	Q_OBJECT
public:

	RestClient(QObject* parent, const QString& base);
	RestClient(const RestClient&) = delete;
	virtual ~RestClient();

	enum class Method {
		GET,
		POST,
		PUT,
		PATCH,
		DELETE
	};

	struct WebException {
		int curlresponse = CURLE_COULDNT_CONNECT;

		WebException(int c = CURLE_COULDNT_CONNECT);
	};

protected:
	QByteArray requestRaw(const QString& url, Method m = Method::GET, const QString& data = {});
	QJsonDocument request(const QString& url, Method m = Method::GET, const QString& data = {});
	void enableDebugging(bool enabled = true);

	QT_DEPRECATED
	QString escape(const QString& in) const;

	QString baseurl;

	QNetworkAccessManager* net;

	QString basicauth; // basic auth code (user:pw) or empty string to disable
#ifdef WIN32
    bool disableCertCheck = true;
#else
    bool disableCertCheck = false;
#endif
	bool debugging = false;
};
